package com.huifu.adapay;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ForegroundColorSpan;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.PopupWindow;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSON;
import com.chinapnr.android.adapay.AdaPay;
import com.chinapnr.android.adapay.AdapayResultActivity;
import com.chinapnr.android.adapay.PayCallback;
import com.chinapnr.android.adapay.bean.PayResult;
import com.chinapnr.android.adapay.bean.PayType;
import com.chinapnr.android.adapay.bean.Payment;
import com.huifu.adapay.beans.PayTypeBean;
import com.huifu.adapay.beans.respbean.BaseRespBean;
import com.huifu.adapay.network.PayNetWorkCallback;
import com.huifu.adapay.widgets.CusListView;
import com.huifu.adapay.widgets.DialogManager;
import com.huifu.adapay.widgets.PayTypeListAdapter;
import com.tencent.mm.opensdk.modelbiz.WXLaunchMiniProgram;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;

import java.util.ArrayList;
import java.util.List;

public class AppPayActivity extends AdapayResultActivity implements View.OnClickListener {

    private String payType;
    private PopupWindow payTypeWindow;

    private TextView tvAmount;
    private TextView tvPayType;
    private TextView tvDoPay;
    //第一次显示界面，不发起支付请求
    private boolean isFirstInit;

    //标识此页面是否走过 onStop 生命周期
    private boolean isStop;

    private PayResult payResult;
    private Payment paymentObj;
    //获取订单信息回调
    private PayNetWorkCallback callback = new PayNetWorkCallback<BaseRespBean>() {
        @Override
        public void onSuccess(BaseRespBean respObj) {

            if (respObj == null) {
                DialogManager.dismiss();
                Toast.makeText(AppPayActivity.this, "支付信息为空", Toast.LENGTH_SHORT).show();
                return;
            }

            paymentObj = JSON.parseObject(respObj.getData(), Payment.class);

            if (isFirstInit) {
                DialogManager.dismiss();
                isFirstInit = false;

                if (paymentObj == null) {
                    DialogManager.dismiss();
                    Toast.makeText(AppPayActivity.this, "支付信息为空", Toast.LENGTH_SHORT).show();
                    return;
                }

                if (!"succeeded".equals(paymentObj.getStatus())) {
                    DialogManager.dismiss();
                    Toast.makeText(AppPayActivity.this, paymentObj.getError_msg(), Toast.LENGTH_SHORT).show();
                    return;
                }

                Spannable string = new SpannableString("总计：¥ " + paymentObj.getPay_amt());
                string.setSpan(new ForegroundColorSpan(Color.RED), 3, string.length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
                tvAmount.setText(string);
            } else {

                AdaPay.doPay(AppPayActivity.this, 60, respObj.getData(), new PayCallback() {
                    @Override
                    public void onPayment(PayResult payResult) {

                        DialogManager.dismiss();
                        //由于支付时离开了页面这里先保存支付结果
                        AppPayActivity.this.payResult = payResult;

                        if (isStop) {
                            return;
                        }

                        startToResultAvtivity();
                    }
                });
            }
        }

        @Override
        public void onFail(BaseRespBean respObj) {
            DialogManager.dismiss();
            Toast.makeText(AppPayActivity.this, "支付失败", Toast.LENGTH_SHORT).show();
        }

        @Override
        public void onFail(String respMsg) {
            DialogManager.dismiss();
            Toast.makeText(AppPayActivity.this, respMsg, Toast.LENGTH_SHORT).show();
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_app_pay);

        initView();

        isFirstInit = true;
        //获取订单信息
        TradeRequest.requestPayment(this, PayType.ALIPAY, callback);
    }

    private void initView() {

        findViewById(R.id.iv_back).setOnClickListener(this);
        ((TextView) findViewById(R.id.tv_title)).setText("订单支付");

        tvAmount = findViewById(R.id.tv_amount);
        tvPayType = findViewById(R.id.tv_choose_pay_method);
        tvDoPay = findViewById(R.id.tv_do_pay);
        findViewById(R.id.layout_choose_method).setOnClickListener(this);

        tvAmount.setOnClickListener(this);
        tvDoPay.setOnClickListener(this);
    }

    @Override
    protected void onStart() {
        super.onStart();
        isStop = false;

        if (payResult != null) {
            startToResultAvtivity();
        }

    }

    private void startToResultAvtivity() {
        Intent intent = new Intent(AppPayActivity.this, PayResultActivity.class);
        intent.putExtra(PayResultActivity.PAYMENT_INFO, paymentObj);
        intent.putExtra(PayResultActivity.TRADE_CODE, payResult.getResultCode());
        intent.putExtra(PayResultActivity.TRADE_MSG, payResult.getResultMsg());
        startActivity(intent);
        finish();
    }

    /**
     * 显示付款方式的悬浮窗
     */
    private void showPopWindow() {
        @SuppressLint("InflateParams") View view = LayoutInflater.from(this).inflate(R.layout.pay_type_window_layout, null, false);

        initPopWindowContent(view);
        //1.构造一个PopupWindow，参数依次是加载的View，宽高
        payTypeWindow = new PopupWindow(view,
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT, true);


        view.measure(View.MeasureSpec.UNSPECIFIED, View.MeasureSpec.UNSPECIFIED);
        int popupHeight = view.getMeasuredHeight();
        int popupWidth = view.getMeasuredWidth();
        int[] location = new int[2];
        tvAmount.getLocationOnScreen(location);
        //在控件上方显示
        payTypeWindow.showAtLocation(tvAmount, Gravity.NO_GRAVITY, (location[0]) - popupWidth / 2, location[1] - popupHeight);
    }

    private List<PayTypeBean> payTypeList;

    private void initPopWindowContent(View view) {
        if (payTypeList == null) {
            PayTypeBean aliPay = new PayTypeBean("支付宝", PayType.ALIPAY, R.mipmap.alipay_logo);
            PayTypeBean wxPay = new PayTypeBean("微信", PayType.WX, R.mipmap.wechat_logo);
            PayTypeBean unionPay = new PayTypeBean("云闪付", PayType.UNION, R.mipmap.union_logo);

            payTypeList = new ArrayList<>();
            payTypeList.add(aliPay);
            payTypeList.add(wxPay);
            payTypeList.add(unionPay);
        }

        CusListView payTypeListView = view.findViewById(R.id.list_pay_type);
        final PayTypeListAdapter payTypeAdapter = new PayTypeListAdapter(this, payTypeList);
        payTypeListView.setAdapter(payTypeAdapter);

        payTypeListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                for (int index = 0; index < payTypeList.size(); index++) {
                    PayTypeBean payTypeBean = payTypeList.get(index);
                    if (index != i) {
                        payTypeBean.setChecked(false);
                        continue;
                    }

                    tvPayType.setText(payTypeBean.getPayTypeName());
                    payType = payTypeBean.getPayType();
                    payTypeBean.setChecked(true);
                    tvDoPay.setEnabled(true);
                    tvDoPay.setBackgroundColor(getResources().getColor(R.color.red));
                    payTypeWindow.dismiss();
                }

                payTypeAdapter.notifyDataSetChanged();

            }
        });
    }

    @Override
    public void onClick(View view) {

        int viewId = view.getId();

        switch (viewId) {
            case R.id.iv_back:
                finish();
                break;

            case R.id.layout_choose_method:
                showPopWindow();
                break;

            case R.id.tv_do_pay:

//                startActivity(new Intent(this, WXEntryActivity.class));

                if (PayType.WX.equals(payType)) {
                    startWechatPay();
                    finish();
                    return;
                }

                //获取订单信息
                TradeRequest.requestPayment(this, payType, callback);
                break;
        }
    }

    private void startWechatPay() {
        IWXAPI api = WXAPIFactory.createWXAPI(this, Config.WECHAT_APPID, false);
        api.registerApp(Config.WECHAT_APPID);

        WXLaunchMiniProgram.Req req = new WXLaunchMiniProgram.Req();
        // 填小程序原始id
        req.userName = "gh_4f60c1629c27";
        //拉起小程序页面的可带参路径，不填默认拉起小程序首页，对于小游戏，可以只传入 query 部分，来实现传参效果，如：传入 "?foo=bar"。
        req.path = "/pages/testPay/testPay?goodstitle=test&amt=0.01&goodsdesc=test";
        req.miniprogramType = WXLaunchMiniProgram.Req.MINIPROGRAM_TYPE_PREVIEW;// 可选打开 开发版，体验版和正式版
        api.sendReq(req);
    }

    @Override
    protected void onStop() {
        super.onStop();

        isStop = true;
    }
}
